﻿#pragma once

#include "../ZegoExpressInterface.h"
#include "../ZegoExpressDefines.h"
#include "../ZegoExpressEventHandler.h"

#include "ZegoInternalBase.h"
#include "ZegoInternalBridge.h"

namespace ZEGO
{
    namespace EXPRESS
    {

        class ZegoExpressMediaPlayerImp : public IZegoMediaPlayer
        {
        public:
            ZegoExpressMediaPlayerImp(int index)
                : instanceIndex(index)
            {
            }

            int getIndex() override
            {
                return instanceIndex;
            }

            void setEventHandler(std::shared_ptr<IZegoMediaPlayerEventHandler> handler) override
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                mpEventHandler = handler;
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            void setVideoHandler(std::shared_ptr<IZegoMediaPlayerVideoHandler> handler, ZegoVideoFrameFormat format) override
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (handler)
                {
                    oInternalOriginBridge->mediaPlayerEnableVideoData(true, zego_video_frame_format(format), zego_mediaplayer_instance_index(instanceIndex));
                }
                else
                {
                    oInternalOriginBridge->mediaPlayerEnableVideoData(false, zego_video_frame_format(format), zego_mediaplayer_instance_index(instanceIndex));
                }
                mpVideoHandler = handler;
            }
#endif

            void setAudioHandler(std::shared_ptr<IZegoMediaPlayerAudioHandler> handler) override
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (handler)
                {
                    oInternalOriginBridge->mediaPlayerEnableAudioData(true, zego_mediaplayer_instance_index(instanceIndex));
                }
                else
                {
                    oInternalOriginBridge->mediaPlayerEnableAudioData(false, zego_mediaplayer_instance_index(instanceIndex));
                }
                mpAudioHandler = handler;
            }

            void loadResource(const std::string &sourcePath, ZegoMediaPlayerLoadResourceCallback callback) override
            {
                int seq = oInternalOriginBridge->mediaPlayerPreload(sourcePath.c_str(), zego_mediaplayer_instance_index(instanceIndex));
                ZEGO_UNUSED_VARIABLE(seq);
                if (callback != nullptr)
                {
                    std::lock_guard<std::mutex> lock(mediaEventMutex);
                    mpLoadResourceCallbacks = callback;
                }
            }

            void start() override
            {
                oInternalOriginBridge->mediaPlayerStart(zego_mediaplayer_instance_index(instanceIndex));
            }

            void stop() override
            {
                oInternalOriginBridge->mediaPlayerStop(zego_mediaplayer_instance_index(instanceIndex));
            }

            void pause() override
            {
                oInternalOriginBridge->mediaPlayerPause(zego_mediaplayer_instance_index(instanceIndex));
            }

            void resume() override
            {
                oInternalOriginBridge->mediaPlayerResume(zego_mediaplayer_instance_index(instanceIndex));
            }

            void seekTo(unsigned long long millisecond, ZegoMediaPlayerSeekToCallback callback) override
            {
                int seq = oInternalOriginBridge->mediaPlayerSeekTo(millisecond, zego_mediaplayer_instance_index(instanceIndex));
                if (callback != nullptr)
                {
                    std::lock_guard<std::mutex> lock(mediaEventMutex);
                    mpSeekToCallbacks.insert({seq, callback});
                }
            }

            void enableRepeat(bool enable) override
            {
                oInternalOriginBridge->mediaPlayerEnableRepeat(enable, zego_mediaplayer_instance_index(instanceIndex));
            }

            void enableAux(bool enable) override
            {
                oInternalOriginBridge->mediaPlayerEnableAUX(enable, zego_mediaplayer_instance_index(instanceIndex));
            }

            void muteLocal(bool mute) override
            {
                oInternalOriginBridge->mediaPlayerMuteLocal(mute, zego_mediaplayer_instance_index(instanceIndex));
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            void setPlayerCanvas(ZegoCanvas *canvas) override
            {
                if (canvas == nullptr)
                {
                    oInternalOriginBridge->mediaPlayerSetPlayerCanvas(nullptr, zego_mediaplayer_instance_index(instanceIndex));
                }
                else
                {
                    zego_canvas _canvas;
                    _canvas.view = canvas->view;
                    _canvas.view_mode = zego_view_mode(canvas->viewMode);
                    _canvas.background_color = canvas->backgroundColor;
                    oInternalOriginBridge->mediaPlayerSetPlayerCanvas(&_canvas, zego_mediaplayer_instance_index(instanceIndex));
                }
            }
#endif

            void setVolume(int volume) override
            {
                oInternalOriginBridge->mediaPlayerSetVolume(volume, zego_mediaplayer_instance_index(instanceIndex));
            }

            void setProgressInterval(unsigned long long millisecond) override
            {
                oInternalOriginBridge->mediaPlayerSetProgressInterval(millisecond, zego_mediaplayer_instance_index(instanceIndex));
            }

            unsigned long long getTotalDuration() override
            {
                return oInternalOriginBridge->mediaPlayerGetTotalDuration(zego_mediaplayer_instance_index(instanceIndex));
            }

            unsigned long long getCurrentProgress() override
            {
                return oInternalOriginBridge->mediaPlayerGetCurrentProgress(zego_mediaplayer_instance_index(instanceIndex));
            }

            int getVolume() override
            {
                return oInternalOriginBridge->mediaPlayerGetVolume(zego_mediaplayer_instance_index(instanceIndex));
            }

            ZegoMediaPlayerState getCurrentState() override
            {
                zego_mediaplayer_state state = oInternalOriginBridge->mediaPlayerGetCurrentState(zego_mediaplayer_instance_index(instanceIndex));
                return ZegoMediaPlayerState(state);
            }

            void zego_on_mediaplayer_state_update(zego_mediaplayer_state state, zego_error error_code)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (mpEventHandler)
                {
                    auto weakMPEventHandler = std::weak_ptr<IZegoMediaPlayerEventHandler>(mpEventHandler);
                    ZEGO_SWITCH_THREAD_PRE
                    auto mpEventHandler = weakMPEventHandler.lock();
                    if (mpEventHandler)
                    {
                        mpEventHandler->onMediaPlayerStateUpdate(this, ZegoMediaPlayerState(state), error_code);
                    }
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            void zego_on_mediaplayer_network_event(zego_mediaplayer_network_event event)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                auto weakMPEventHandler = std::weak_ptr<IZegoMediaPlayerEventHandler>(mpEventHandler);
                ZEGO_SWITCH_THREAD_PRE
                auto mpEventHandler = weakMPEventHandler.lock();
                if (mpEventHandler)
                {
                    mpEventHandler->onMediaPlayerNetworkEvent(this, ZegoMediaPlayerNetworkEvent(event));
                }
                ZEGO_SWITCH_THREAD_ING
            }

            void zego_on_mediaplayer_playing_progress(unsigned long long millisecond)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                auto weakMPEventHandler = std::weak_ptr<IZegoMediaPlayerEventHandler>(mpEventHandler);
                ZEGO_SWITCH_THREAD_PRE
                auto mpEventHandler = weakMPEventHandler.lock();
                if (mpEventHandler)
                {
                    mpEventHandler->onMediaPlayerPlayingProgress(this, millisecond);
                }
                ZEGO_SWITCH_THREAD_ING
            }

            void zego_on_mediaplayer_seek_to_result(zego_seq seq, zego_error error_code)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (mpSeekToCallbacks.count(seq) > 0)
                {
                    auto cb = mpSeekToCallbacks[seq];
                    mpSeekToCallbacks.erase(seq);
                    ZEGO_SWITCH_THREAD_PRE
                    cb(error_code);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            void zego_on_mediaplayer_load_file_result(zego_error error_code)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (mpLoadResourceCallbacks)
                {
                    ZEGO_SWITCH_THREAD_PRE
                    mpLoadResourceCallbacks(error_code);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            void zego_on_mediaplayer_video_data(const unsigned char **data, unsigned int *data_length, const struct zego_video_frame_param _param)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (mpVideoHandler)
                {
                    auto param = ZegoExpressConvert::I2OVideoFrameParam(_param);
                    mpVideoHandler->onVideoFrame(this, data, data_length, param);
                }
            }
#endif

            void zego_on_mediaplayer_audio_data(const unsigned char *data, unsigned int data_length, const struct zego_audio_frame_param _param)
            {
                std::lock_guard<std::mutex> lock(mediaEventMutex);
                if (mpAudioHandler)
                {
                    ZegoAudioFrameParam param;
                    param.channel = ZegoAudioChannel(_param.channel);
                    param.sampleRate = ZegoAudioSampleRate(_param.samples_rate);
                    mpAudioHandler->onAudioFrame(this, data, data_length, param);
                }
            }

        private:
#if ZEGO_EXPRESS_VIDEO_SDK
            std::shared_ptr<IZegoMediaPlayerVideoHandler> mpVideoHandler;
#endif
            std::shared_ptr<IZegoMediaPlayerEventHandler> mpEventHandler;
            std::shared_ptr<IZegoMediaPlayerAudioHandler> mpAudioHandler;
            ZegoMediaPlayerLoadResourceCallback mpLoadResourceCallbacks;
            std::unordered_map<zego_seq, ZegoMediaPlayerSeekToCallback> mpSeekToCallbacks;
            std::mutex mediaEventMutex;
            int instanceIndex = -1;
        };

    } // namespace EXPRESS
} // namespace ZEGO