#ifndef __ZEGO_EXPRESS_DEVICE_H__
#define __ZEGO_EXPRESS_DEVICE_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 音频设备状态变更事件
 * 
 * @param update_type 音频设备的更新类型，添加/删除
 * @param device_type 音频设备类型
 * @param device_info 音频设备信息
 * @param user_context 用户上下文
 */
typedef void(*zego_on_audio_device_state_changed)(enum zego_update_type update_type, enum zego_audio_device_type device_type, struct zego_device_info device_info, void* user_context);

/**
 * 视频设备状态变更事件
 * 针对于 Mac、Windows 系统上视频设备的插入和拔除事件通知
 * @param update_type 视频设备的更新类型，添加/删除
 * @param device_info 视频设备信息
 * @param user_context 用户上下文
 */
typedef void(*zego_on_video_device_state_changed)(enum zego_update_type update_type, struct zego_device_info device_info, void* user_context);

/**
 * 本地采集音频声浪回调
 * 
 * @param sound_level_info 本地采集的声浪信息
 * @param user_context 用户上下文
 */
typedef void(*zego_on_captured_sound_level_update)(const struct zego_sound_level_info* sound_level_info, void* user_context);

/**
 * 远端音频声浪回调
 * 
 * @param sound_level_info_list 远端的声浪信息列表
 * @param info_count 远端的声浪信息列表数量
 * @param user_context 用户上下文
 */
typedef void(*zego_on_remote_sound_level_update)(const struct zego_sound_level_info* sound_level_info_list, unsigned int info_count, void* user_context);

/**
 * 本地采集音频频谱回调
 * 
 * @param audio_spectrum_info 本地采集的音频频谱信息
 * @param user_context 用户上下文
 */
typedef void(*zego_on_captured_audio_spectrum_update)(const struct zego_audio_spectrum_info* audio_spectrum_info, void* user_context);

/**
 * 远端拉流音频频谱回调
 * 
 * @param audio_spectrum_info_list 远端的音频频谱信息列表
 * @param info_count 远端的音频频谱信息列表数量
 * @param user_context 用户上下文
 */
typedef void(*zego_on_remote_audio_spectrum_update)(const struct zego_audio_spectrum_info* audio_spectrum_info_list, unsigned int info_count, void* user_context);

/**
 * 设备异常通知
 * 当本地设备读写出现异常时会触发此回调。
 * @param error_code 错误码
 * @param device_name 设备名称
 * @param user_context 用户上下文
 */
typedef void(*zego_on_device_error)(zego_error error_code, const char* device_name, void* user_context);

/**
 * 远端摄像头设备状态通知
 * 远端摄像头设备状态发生变更时，例如开关摄像头等，通过监听此回调，能够获取远端摄像头相关的事件，可以用于提示用户可能导致视频异常的情况。
 * @param stream_id 流 ID
 * @param state 远端摄像头状态
 * @param user_context 用户上下文
 */
typedef void(*zego_on_remote_camera_state_update)(const char* stream_id, enum zego_remote_device_state state, void* user_context);

/**
 * 远端麦克风设备状态通知
 * 
 * @param stream_id 流 ID
 * @param state 远端麦克风状态
 * @param user_context 用户上下文
 */
typedef void(*zego_on_remote_mic_state_update)(const char* stream_id, enum zego_remote_device_state state, void* user_context);

/**
 * 是否关闭麦克风输入
 * 此接口用于控制是否使用采集到的音频数据，关闭麦克风即采集到数据后就丢弃，此时仍然会占用麦克风
 * @param mute 是否关闭麦克风输入。true 表示关闭麦克风；false 表示开启麦克风。
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_microphone(bool mute);

/**
* 获取当前麦克风是否关闭输入的状态
*
* @return (bool), 是否关闭输入
*/
ZEGOEXP_API bool EXP_CALL zego_express_get_microphone_mute();

/**
 * 是否静音音频输出（废弃）
 * 接口已废弃，请使用 [zego_express_mute_speaker]
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_audio_output(bool mute);

/**
 * 是否关闭音频输出
 * 关闭后SDK将不会再传递音频数据给输出设备，以达到静音的效果
 * @param mute 是否关闭音频输出，true表示关闭音频输出；false 表示开启音频输出
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_speaker(bool mute);

/**
* 获取当前音频输出是否静音的状态
*
* @return (bool), 是否静音
*/
ZEGOEXP_API bool EXP_CALL zego_express_get_speaker_mute();

/**
 * 选择使用某个音频设备
 * 只适用于 Mac、Windows 系统
 * @param device_type 音频设备类型，输入/输出设备
 * @param device_id 音频设备ID
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_use_audio_device(enum zego_audio_device_type device_type, const char* device_id);

/**
 * 获取当前系统上的音频设备列表
 * 只适用于 Mac、Windows 系统
 * @param device_type 音频设备类型，输入/输出设备
 * @param device_count 音频设备数量
 * @return (struct zego_device_info*), 音频设备列表
 */
ZEGOEXP_API struct zego_device_info* EXP_CALL zego_express_get_audio_device_list(enum zego_audio_device_type device_type, int* device_count);

/**
 * 释放音频设备列表
 * 只适用于 Mac、Windows 系统
 * @param device_list 音频设备列表
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_free_audio_device_list(struct zego_device_info* device_list);

/**
 * 开/关音频采集设备
 * 此接口用于控制是否释放音频采集设备，关闭音频采集设备，则 SDK 不会再占用音频设备，当然如果此时正在推流，也没有音频数据。
 * @param enable 是否使能音频采集设备，true 表示开启音频采集设备，false 表示关闭音频采集设备
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_audio_capture_device(bool enable);

/**
 * 是否使用内置扬声器播放声音
 *
 * 仅iOS/Android使用。当选择不使用内置扬声器播放声音时，SDK会根据系统调度，选择相对应的音频输出设备播放声音。
 * @param enable 使用内置扬声器开关
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_built_in_speaker_on(bool enable);

/**
 * 开/关摄像头
 * 此接口用于控制是否启动摄像头的采集，关闭摄像头后，将不会进行视频采集，此时推流也将没有视频数据
 * @param enable 是否打开摄像头，false 表示关闭摄像头；true 表示开启摄像头
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_camera(bool enable, enum zego_publish_channel channel);

/**
 * 切换前后摄像头
 * 此接口用于控制使用前摄像头或者后摄像头
 * @param enable 是否采用前置摄像头，false 表示使用后置摄像头；true 表示使用前置摄像头
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_use_front_camera(bool enable, enum zego_publish_channel channel);

/**
 * 设置使用的摄像头设备
 * 只适用于 Mac、Windows 系统
 * @param device_id 摄像头设备ID
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_use_video_device(const char* device_id, enum zego_publish_channel channel);

/**
 * 获取当前系统上的摄像头设备列表
 * 只适用于 Mac、Windows 系统
 * @param device_count 摄像头设备数量
 * @return (struct zego_device_info*), 摄像头设备列表
 */
ZEGOEXP_API struct zego_device_info* EXP_CALL zego_express_get_video_device_list(int* device_count);

/**
 * 释放摄像头设备列表
 * 只适用于 Mac、Windows 系统
 * @param device_list 摄像头设备列表
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_free_video_device_list(struct zego_device_info* device_list);

/**
 * 启动声浪监控
 *
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_start_sound_level_monitor(void);

/**
 * 停止声浪监控
 *
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_stop_sound_level_monitor(void);

/**
 * 启动音频频谱监控
 *
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_start_audio_spectrum_monitor(void);

/**
 * 停止音频频谱监控
 *
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_stop_audio_spectrum_monitor(void);

/**
* 开启/关闭 SDK 采集监听（耳返）
* 推流时可调用本 API 进行参数配置。当连接耳麦时该设置才实际生效。开启采集监听，用户使用麦克风采集声音时会听到自己的声音。
* @param enable 是否开启，默认为 false
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_headphone_monitor(bool enable);

/**
* 设置 SDK 采集监听音量（耳返）
* 推流时可调用本 API 进行参数配置。当连接耳麦时该设置才实际生效。开启采集监听，用户使用麦克风采集声音时会听到自己的声音。
* @param volume 采集监听的音量大小，取值（0, 100）。默认 80
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_set_headphone_monitor_volume(int volume);

/**
* 开启/关闭 系统声卡采集（ 仅支持 windows/macOS ）
* 此 API 可将系统播放的声音混入推流中，如浏览器播放的声音、其他软件播放的声音等等
* @param enable 是否开启，默认为 false
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_mix_system_playout(bool enable);

/**
 * 注册音频设备状态变更事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 音频设备状态变更回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_audio_device_state_changed_callback(zego_on_audio_device_state_changed callback_func, void* user_context);

/**
 * 注册视频设备状态变更事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 视频设备状态变更回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_video_device_state_changed_callback(zego_on_video_device_state_changed callback_func, void* user_context);

/**
 * 注册采集音浪更新事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 采集音浪回调接口
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_captured_sound_level_update_callback(zego_on_captured_sound_level_update callback_func, void* user_context);

/**
 * 注册拉流音浪更新事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 拉流音浪回调接口
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_remote_sound_level_update_callback(zego_on_remote_sound_level_update callback_func, void* user_context);

/**
 * 注册采集频域功率谱更新事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 采集频域功率谱回调接口
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_captured_audio_spectrum_update_callback(zego_on_captured_audio_spectrum_update callback_func, void* user_context);

/**
 * 注册拉流频域功率谱更新事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 拉流频域功率谱回调接口
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_remote_audio_spectrum_update_callback(zego_on_remote_audio_spectrum_update callback_func, void* user_context);

/**
 * 注册设备错误事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 设备错误事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_device_error_callback(zego_on_device_error callback_func, void* user_context);

/**
 * 注册收到远端摄像头状态变更事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 远端摄像头状态变更事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_remote_camera_state_update_callback(zego_on_remote_camera_state_update callback_func, void* user_context);

/**
 * 注册收到远端摄像头状态变更事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 远端摄像头状态变更事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_remote_mic_state_update_callback(zego_on_remote_mic_state_update callback_func, void* user_context);

ZEGO_END_DECLS

#endif
