#ifndef __ZEGO_EXPRESS_IM_H__
#define __ZEGO_EXPRESS_IM_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 发送房间聊天消息结果通知
 * 
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param message_id 消息ID
 * @param error_code 错误码
 * @param seq 请求序列号，与调用zego_express_send_broadcast_message时获得的seq对应
 * @param user_context 用户上下文
 */
typedef void(*zego_on_im_send_broadcast_message_result)(const char* room_id, unsigned long long message_id, zego_error error_code, zego_seq seq, void* user_context);

/**
 * 接收房间聊天消息通知
 * 
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param message_info_list 接收到的消息列表
 * @param message_count 接收到的消息列表个数
 * @param user_context 用户上下文
 */
typedef void(*zego_on_im_recv_broadcast_message)(const char* room_id, const struct zego_broadcast_message_info* message_info_list, unsigned int message_count, void* user_context);

/**
 * 发送房间自定义信令结果通知
 * 
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param error_code 错误码
 * @param seq 请求序列号，与调用zego_express_send_custom_command时获得的seq对应
 * @param user_context 用户上下文
 */
typedef void(*zego_on_im_send_custom_command_result)(const char* room_id, zego_error error_code, zego_seq seq, void* user_context);

/**
 * 接收房间自定义信令通知
 * 
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param from_user 发送的用户
 * @param content 接收到的消息内容
 * @param user_context 用户上下文
 */
typedef void(*zego_on_im_recv_custom_command)(const char* room_id, struct zego_user from_user, const char* content, void* user_context);

/**
 * 发送房间弹幕消息结果通知
 *
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param error_code 错误码
 * @param seq 请求序列号，与调用 zego_express_send_barrage_message 时获得的seq对应
 * @param user_context 用户上下文
 */
typedef void(*zego_on_im_send_barrage_message_result)(const char* room_id, const char* message_id, zego_error error_code, zego_seq seq, void* user_context);

/**
 * 接收房间弹幕消息通知
 *
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param message_info_list 接收到的消息列表
 * @param message_count 接收到的消息列表个数
 * @param user_context 用户上下文
 */
typedef void(*zego_on_im_recv_barrage_message)(const char* room_id, const struct zego_barrage_message_info* message_info_list, unsigned int message_count, void* user_context);

/**
 * 发送房间聊天消息
 *
 * 默认房间人数在500人以内时，房间内用户均能收到聊天信息
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param content 房间消息内容，取值范围：任意字符，内容长度不超过512字节
 * @return (zego_seq), 消息序列号
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_send_broadcast_message(const char* room_id, const char* content);

/**
 * 发送自定义信令
 *
 * @param room_id 房间ID，取值范围：长度不超过128字节的字符串
 * @param content 房间消息内容，to_user_list列表中的用户会收到该消息。取值范围：任意字符，内容长度不超过512字节
 * @param to_user_list 发送消息的用户列表，to_user_list传空或者to_user_count传0代表向房间内所有用户发送消息
 * @param to_user_count 发送消息的用户个数，to_user_list传空或者to_user_count传0代表向房间内所有用户发送消息
 * @return (zego_seq), 消息序列号
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_send_custom_command(const char* room_id, const char* content, struct zego_user*  to_user_list, unsigned int to_user_count);

/**
* 发送房间弹幕消息
*
* 不可靠信道消息，主要用于大并发的场景，发送一些非必须到达的消息。需要在房间超过500人的情况下发送消息，请选择使用发送弹幕消息
* @param room_id 房间ID，取值范围：长度不超过128字节的字符串
* @param content 房间消息内容，取值范围：任意字符，内容长度不超过512字节
* @return (zego_seq), 消息序列号
*/
ZEGOEXP_API zego_seq EXP_CALL zego_express_send_barrage_message(const char* room_id, const char* content);

/**
 * 注册接收房间聊天消息发送结果的回调
 * 
 * @param event_handler 事件通知处理句柄
 * @param callback_func 房间消息发送结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_im_send_broadcast_message_result_callback(zego_on_im_send_broadcast_message_result callback_func, void* user_context);

/**
 * 注册接收房间聊天消息的回调
 * 
 * @param event_handler 事件通知处理句柄
 * @param callback_func 接收房间聊天消息回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_im_recv_broadcast_message_callback(zego_on_im_recv_broadcast_message callback_func, void* user_context);

/**
 * 注册接收房间自定义信令发送结果的回调
 * 
 * @param event_handler 事件通知处理句柄
 * @param callback_func 房间消息发送结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_im_send_custom_command_result_callback(zego_on_im_send_custom_command_result callback_func, void* user_context);

/**
 * 注册接收房间自定义信令的回调
 * 
 * @param event_handler 事件通知处理句柄
 * @param callback_func 接收房间消息回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_im_recv_custom_command_callback(zego_on_im_recv_custom_command callback_func, void* user_context);

/**
 * 注册接收房间聊天消息发送结果的回调
 *
 * @param event_handler 事件通知处理句柄
 * @param callback_func 房间消息发送结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_im_send_barrage_message_result_callback(zego_on_im_send_barrage_message_result callback_func, void* user_context);

/**
 * 注册接收房间聊天消息的回调
 *
 * @param event_handler 事件通知处理句柄
 * @param callback_func 接收房间聊天消息回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_im_recv_barrage_message_callback(zego_on_im_recv_barrage_message callback_func, void* user_context);

ZEGO_END_DECLS

#endif
