#ifndef __ZEGO_EXPRESS_MIXER_H__
#define __ZEGO_EXPRESS_MIXER_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 开始混流结果回调
 * 
 * @param task_id 混流任务ID
 * @param result 混流结果
 * @param seq 混流序列号
 * @param extended_data 扩展内容，
 * @param user_context 用户上下文
 */
typedef void(*zego_on_mixer_start_result)(zego_error error_code, zego_seq seq, const char* extended_data, void* user_context);

/**
 * 停止混流结果回调
 *
 * @param task_id 混流任务ID
 * @param result 混流结果
 * @param seq 混流序列号
 * @param user_context 用户上下文
 */
typedef void(*zego_on_mixer_stop_result)(zego_error error_code, zego_seq seq, void* user_context);

/**
 * 开始自动混流结果回调
 *
 * @param task_id 混流任务ID
 * @param result 混流结果
 * @param seq 混流序列号
 * @param user_context 用户上下文
 */
typedef void(*zego_on_auto_mixer_start_result)(zego_error error_code, zego_seq seq, const char* extended_data, void* user_context);

/**
 * 停止混流结果回调
 *
 * @param task_id 混流任务ID
 * @param result 混流结果
 * @param seq 混流序列号
 * @param user_context 用户上下文
 */
typedef void(*zego_on_auto_mixer_stop_result)(zego_error error_code, zego_seq seq, void* user_context);

/**
* 混流音浪更新回调
*
* @param sound_level_info_list 音浪列表
* @param info_count 音浪数
* @param user_context 用户上下文
*/
typedef void(*zego_on_mixer_sound_level_update)(const struct zego_mixer_sound_level_info* sound_level_info_list, unsigned int info_count, void* user_context);

/**
* 自动混流音浪更新回调
*
* @param sound_level_info_list 音浪列表
* @param info_count 音浪数
* @param user_context 用户上下文
*/
typedef void(*zego_on_auto_mixer_sound_level_update)(const struct zego_mixer_sound_level_info* sound_level_info_list, unsigned int info_count, void* user_context);

/**
 * 混流转推CDN状态回调
 * 
 * @param task_id 混流任务ID
 * @param cdn_info_list 混流转推CDN状态列表
 * @param info_count 混流转推CDN状态列表个数
 * @param user_context 用户上下文
 */
typedef void(*zego_on_mixer_relay_cdn_state_update)(const char* task_id, const struct zego_stream_relay_cdn_info* cdn_info_list, unsigned int info_count, void* user_context);

/**
 * 开始混流
 *
 * @param task  混流任务配置
 * @return (zego_seq), 混流发起序列号
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_start_mixer_task(struct zego_mixer_task task);

/**
 * 停止混流
 *
 * @param task_id 混流任务ID
 * @return (zego_error), 混流停止序列号
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_stop_mixer_task(struct zego_mixer_task task);

/**
 * 开始自动混流，仅支持音频流的自动混流
 *
 * 自动混流需要与房间绑定，当开启自动混流后，指定房间内的用户推流成功后自动进行服务端混流
 * @param task  自动混流任务配置
 * @return (zego_seq), 混流发起序列号
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_start_auto_mixer_task(struct zego_auto_mixer_task task);

/**
 * 停止自动混流
 *
 * @param task_id 混流任务ID
 * @param room_id 房间ID
 * @return (zego_error), 混流停止序列号
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_stop_auto_mixer_task(struct zego_auto_mixer_task task);

/**
 * 注册混流任务开始事件回调
 * 
 * @param event_handler 事件通知处理句柄
 * @param callback_func 开始混流结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_mixer_start_result_callback(zego_on_mixer_start_result callback_func, void* user_context);

/**
 * 注册混流任务停止事件回调
 *
 * @param event_handler 事件通知处理句柄
 * @param callback_func 停止混流结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_mixer_stop_result_callback(zego_on_mixer_stop_result callback_func, void* user_context);

/**
 * 注册自动音频混流任务开始事件回调
 *
 * @param event_handler 事件通知处理句柄
 * @param callback_func 开始混流结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_auto_mixer_start_result_callback(zego_on_auto_mixer_start_result callback_func, void* user_context);

/**
 * 注册自动音频混流任务停止事件回调
 *
 * @param event_handler 事件通知处理句柄
 * @param callback_func 停止混流结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_auto_mixer_stop_result_callback(zego_on_auto_mixer_stop_result callback_func, void* user_context);

/**
* 注册混流音浪更新回调
*
* @param event_handler 事件通知处理句柄
* @param callback_func 混流音浪更新回调
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_mixer_sound_level_update(zego_on_mixer_sound_level_update callback_func, void* user_context);

/**
* 注册自动混流音浪更新回调
*
* @param event_handler 事件通知处理句柄
* @param callback_func 自动混流音浪更新回调
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_auto_mixer_sound_level_update(zego_on_auto_mixer_sound_level_update callback_func, void* user_context);

/**
 * 注册混流转推CDN状态回调
 * 
 * @param event_handler 事件通知处理句柄
 * @param callback_func 混流转推CDN状态回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_mixer_relay_cdn_state_update_callback(zego_on_mixer_relay_cdn_state_update callback_func, void* user_context);


ZEGO_END_DECLS

#endif
