#ifndef __ZEGO_EXPRESS_PLAYER_H__
#define __ZEGO_EXPRESS_PLAYER_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 拉流状态变更回调
 * 在拉流成功后，可以通过该回调接口获取拉流状态变更的通知。
 * @param stream_id 流 ID
 * @param state 拉流状态
 * @param error_code 拉流状态变更对应的错误码。请参考常见错误码文档
 * @param extend_data 扩展内容，默认为空
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_state_update)(const char* stream_id, enum zego_player_state state, zego_error error_code, const char* extended_data, void* user_context);

/**
 * 拉流质量回调
 * 拉流成功后每3秒会收到此回调，通过该回调可以获取拉取的音视频流的帧率，码率，RTT，丢包率等质量数据，实时监控拉取流的健康情况。
 * @param stream_id 流 ID
 * @param quality 拉流质量，包含了音视频帧率、码率、分辨率，RTT等值
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_quality_update)(const char* stream_id, struct zego_play_stream_quality quality, void* user_context);

/**
 * 流媒体事件通知
 * 当拉流发生音视频卡顿以及恢复等事件发生时会触发此回调。
 * @param stream_id 流 ID
 * @param event 拉流媒体事件枚举
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_media_event)(const char* stream_id, enum zego_player_media_event event, void* user_context);

/**
 * 收到远端音频首帧回调
 * 调用推流接口成功后，当拉取到音频首帧、视频首帧以及视频首帧被渲染都会触发此回调。
 * @param stream_id 流 ID
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_recv_audio_first_frame)(const char* stream_id, void* user_context);

/**
 * 收到远端视频首帧回调
 * 
 * @param stream_id 流 ID
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_recv_video_first_frame)(const char* stream_id, void* user_context);

/**
 * 渲染远端视频首帧回调
 * 
 * @param stream_id 流 ID
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_render_video_first_frame)(const char* stream_id, void* user_context);

/**
 * 拉流分辨率变更通知
 * 拉流成功后，在拉流中途如果有视频分辨率发生变化将会收到此回调，用户可根据流的最终分辨率调整显示。
 * @param stream_id 流 ID
 * @param width 视频分辨率宽
 * @param height 视频分辨率高
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_video_size_changed)(const char* stream_id, int width, int height, void* user_context);

/**
 * 收到远端流的SEI内容
 * 拉流成功后，当远端流调用 zego_express_send_sei 后，本端会收到此回调
 * @param stream_id 流 ID
 * @param data SEI内容
 * @param data_length SEI内容长度
 * @param user_context 用户上下文
 */
typedef void(*zego_on_player_recv_sei)(const char* stream_id, const unsigned char* data, unsigned int data_length, void* user_context);

/**
 * 开始拉流
 * 可通过此接口让用户可以从 ZEGO 实时音视频云拉取远端用户的音视频流进行互通。
 * 在开始拉流前，需要先加入房间，可通过监听 zego_on_room_stream_update 事件回调来获取该房间内 streamID 的新增。
 * 在网络质量不佳的情况下，用户拉流可能出现中断， SDK 会尝试重新连接，可通过监听 zego_on_room_state_update 事件来获知当前拉流状态以及错误信息。
 * 拉取不存在的流 ID，执行本接口后 SDK 持续尝试拉取，在该流 ID被成功推送后，音视频流可以真正被拉取到。
 * 开发者可通过重复调用此接口实现更换拉流Canvas的操作（流ID必须一样）。
 * @param stream_id 流 ID，长度不超过256的字符串。
 * @param canvas 用于显示拉流画面的视图，视图设置为空则不进行显示。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_start_playing_stream(const char* stream_id, struct zego_canvas* canvas);

/**
* 开始拉流
* 可通过此接口让用户可以从第三方URL拉取远端用户的音视频流。
* 在开始拉流前，需要先加入房间，可通过监听 zego_on_room_stream_update 事件回调来获取该房间内 streamID 的新增。
* 在网络质量不佳的情况下，用户拉流可能出现中断， SDK 会尝试重新连接，可通过监听 zego_on_room_state_update 事件来获知当前拉流状态以及错误信息。
* 拉取不存在的流 ID，执行本接口后 SDK 持续尝试拉取，在该流 ID被成功推送后，音视频流可以真正被拉取到。
* 开发者可通过重复调用此接口实现更换拉流Canvas的操作（流ID必须一样）。
* @param stream_id 流 ID，长度不超过256的字符串。
* @param canvas 用于显示拉流画面的视图，视图设置为空则不进行显示。
* @return (zego_error)
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_start_playing_stream_with_config(const char* stream_id, struct zego_canvas* canvas, struct zego_player_config config);

/**
 * 停止拉流
 * 
 * @param stream_id 流 ID，长度不超过256的字符串
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_stop_playing_stream(const char* stream_id);

/**
 * 设置拉流的音量
 * 此接口用于设置拉流的声音大小，本端用户可控制音频流的播放音量。需要在调用拉流接口后调用。
 * @param stream_id 流 ID，长度不超过256的字符串
 * @param volume 音量百分比
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_play_volume(const char* stream_id, int volume);

/**
 * 停止或恢复拉取音频流
 * 拉流时可调用此接口实现不拉取远端用户的音频数据，需要在调用拉流接口后调用。
 * @param stream_id 流 ID，长度不超过256的字符串
 * @param mute 禁用标识。true:禁止拉取；false:恢复拉取
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_play_stream_audio(const char* stream_id, bool mute);

/**
 * 停止或恢复拉取视频流
 * 拉流时可调用此接口实现不拉取远端用户的视频数据，需要在调用拉流接口后调用。
 * @param stream_id 流 ID，长度不超过256的字符串。
 * @param mute 禁用标识。true:禁止拉取；false:恢复拉取
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_play_stream_video(const char* stream_id, bool mute);

/**
 * 开/关硬件解码
 * 拉流时是否采用硬件解码的开关，开启硬解解码后会使用GPU进行解码，降低CPU使用率。在拉流前设置才能生效，如果在拉流后设置，停止拉流后重新拉流可以生效。
 * @param enable 是否开启硬解开关。true:启用硬解；false:关闭硬解
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_hardware_decoder(bool enable);

/**
 * 帧顺序检测开关
 *
 * @param enable 是否检测帧顺序，true 表示检测，不支持B帧；false 表示不检测，支持B帧但可能出现短暂花屏
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_check_poc(bool enable);

/**
 * 注册拉流状态事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 拉流状态事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_state_update_callback(zego_on_player_state_update callback_func, void* user_context);

/**
 * 注册拉流质量更新事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 拉流质量更新回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_quality_update_callback(zego_on_player_quality_update callback_func, void* user_context);

/**
 * 注册媒体事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 拉流媒体事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_media_event_callback(zego_on_player_media_event callback_func, void* user_context);

/**
 * 注册接收音频首帧事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 音频首帧事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_recv_audio_first_frame_callback(zego_on_player_recv_audio_first_frame callback_func, void* user_context);

/**
 * 注册接收视频首帧事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 视频首帧事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_recv_video_first_frame_callback(zego_on_player_recv_video_first_frame callback_func, void* user_context);

/**
 * 注册收到远端首帧视频渲染完成事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 远端首帧视频渲染完成事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_render_video_first_frame_callback(zego_on_player_render_video_first_frame callback_func, void* user_context);

/**
 * 注册收到远端视频大小变更事件回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 远端视频大小变更事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_video_size_changed_callback(zego_on_player_video_size_changed callback_func, void* user_context);

/**
 * 注册收到远端流SEI内容回调
 *
 * @param callback_func 远端流SEI内容回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_player_recv_sei_callback(zego_on_player_recv_sei callback_func, void* user_context);

ZEGO_END_DECLS

#endif
