#ifndef __ZEGO_EXPRESS_PREPROCESS_H__
#define __ZEGO_EXPRESS_PREPROCESS_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

typedef void(*zego_on_copy_audio_mixing_data)(struct zego_audio_mixing_data *data, void* user_context);


/**
 * 设置回声消除开关
 * 打开回声消除， SDK 会对采集到的音频数据进行过滤以降低音频中的回音成分。需要在开始推流之前设置，推流开始后设置无效。
 * @param enable 是否开启回声消除，false 表示关闭回声消除，true 表示开启回声消除
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_aec(bool enable);

/**
 * 是否在使用耳机时开启回声消除
 * 需要在推流或预览之前调用才有效
 * 打开回声消除， SDK 会对采集到的音频数据进行过滤以降低音频中的回音成分。需要在开始推流之前设置，推流开始后设置无效。iOS 平台默认为 true，android 平台默认为 false
 * @param enable 是否在插入耳机后开启回声消除，false 表示关闭回声消除，true 表示开启回声消除
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_headphone_aec(bool enable);

/**
 * 设置回声消除模式
 * 切换不同的回声消除模式以控制消除回声数据的程度。需要在开始推流之前设置。
 * @param mode 回声消除模式
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_aec_mode(enum zego_aec_mode mode);

/**
 * 开/关自动增益
 * 开启自动增益后声音会被放大，但会一定程度上影响音质。需要在开始推流之前设置。
 * @param enable 是否开启自动增益，false 表示关闭自动增益，true 表示开启自动增益
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_agc(bool enable);

/**
 * 开/关噪声抑制
 * 开启噪声抑制可以使人声更加清晰。需要在开始推流之前设置，推流开始后设置无效。
 * @param enable 是否开启噪声抑制，false 表示关闭噪声抑制，true 表示开启噪声抑制
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_ans(bool enable);

/**
 * 设置音频采集噪声抑制等级
 * 当开发者调用了 [zego_express_enable_ans] 开启噪声抑制之后，此接口才能生效。默认为 zego_ans_mode_medium
 * @param mode 降噪等级，详见 zego_ans_mode 定义
 * @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_set_ans_mode(enum zego_ans_mode mode);

/**
* 开启/关闭混音
* 当开启混音后，SDK 会在 zego_on_copy_aux_data 中获取用于混音的音频数据，即需要开发者在此回调中修改参数将音频数据传给 SDK。
* @param enable 是否开启混音，false 表示关闭，true 表示开启，默认为 false
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_audio_mixing(bool enable);

/**
* 混音本地静音开关
* 当开启本地混音的静音后，混音端（推流端）将听不到混音的播放声音，远端（拉流端）依然能听到混音
* @param enable 是否将混音静音，true 表示静音，false 表示不静音，默认为 false
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_local_audio_mixing(bool mute);

/**
* 设置混音音量
* 此 API 会同时修改本地播放以及混到推流中的混音数据音量大小，若需要单独设置本地播放的混音音量或混入推流中的混音音量，请使用 zego_express_set_aux_volume_with
* @param volume 音量，取值范围是 0 ~ 100，默认为 50
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_set_audio_mixing_volume(int volume);

/**
* 设置混音音量
* 此 API 可以单独设置本地播放的混音音量或混入推流中的混音音量
* @param volume 音量，取值范围是 0 ~ 100，默认为 50
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_set_audio_mixing_volume_with_type(int volume, enum zego_volume_type type);

/**
 * 开/关美颜
 * 识别视频中的人像进行美颜。开始推流前后都可以设置。
 * @param feature_bitmask 比特掩码格式，可选择同时开启 ZegoBeautifyFeature 中的几个特性
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_beautify(int feature_bitmask, enum zego_publish_channel channel);

/**
 * 设置美颜参数
 * 
 * @param option 美颜选项参数
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_beautify_option(struct zego_beautify_option option, enum zego_publish_channel channel);

ZEGOEXP_API void EXP_CALL zego_register_audio_mixing_copy_data_callback(zego_on_copy_audio_mixing_data callback_func, void* user_context);

ZEGOEXP_API zego_error EXP_CALL zego_express_set_voice_changer_param(float param);

ZEGOEXP_API zego_error EXP_CALL zego_express_enable_virtual_stereo(bool enable, int angle);

ZEGOEXP_API zego_error EXP_CALL zego_express_set_reverb_param(struct zego_reverb_param param);


ZEGO_END_DECLS

#endif
