#ifndef __ZEGO_EXPRESS_PUBLISHER_H__
#define __ZEGO_EXPRESS_PUBLISHER_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 推流状态回调
 * 在推流成功后，可以通过该回调接口获取推流状态变更的通知。
 * @param stream_id 推流的流 ID
 * @param state 推流状态
 * @param error_code 推流状态变更对应的错误码。请参考常见错误码文档
 * @param extend_data 扩展内容，默认为空
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_state_update)(const char* stream_id, enum zego_publisher_state state, zego_error error_code, const char* extended_data, void* user_context);

/**
 * 推流质量回调
 * 推流成功后每3秒会收到此回调，通过该回调可以获取推送的音视频流的采集帧率，码率，RTT，丢包率等质量数据，实时监控推送流的健康情况。
 * @param stream_id 流 ID
 * @param quality 流质量，包含了音视频帧率、码率、分辨率，RTT等值
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_quality_update)(const char* stream_id, struct zego_publish_stream_quality quality, void* user_context);

/**
 * 流附加信息更新结果回调
 * 
 * @param error_code 错误码
 * @param seq 请求序列号，与调用zego_express_update_publish_stream_extra_info 时获得的seq对应
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_update_stream_extra_info_result)(zego_error error_code, zego_seq seq, void* user_context);

/**
 * 推流端采集音频首帧回调
 * 
 * 调用推流接口成功后，SDK 采集到第一帧数据时会收到此回调，包含了音频/视频的采集首帧通知。
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_recv_audio_captured_first_frame)(void* user_context);

/**
 * 推流端采集视频首帧回调
 * 推流成功后会收到此回调
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_recv_video_captured_first_frame)(enum zego_publish_channel channel, void* user_context);

/**
 * 采集视频大小变更回调
 * 推流成功后，在推流中途如果有改变视频采集分辨率发生变化将会收到此回调。
 * @param width 视频采集分辨率宽
 * @param height 视频采集分辨率高
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_video_size_changed)(int width, int height, enum zego_publish_channel channel, void* user_context);

/**
 * 添加/删除转推CDN地址回调
 * 调用 zego_express_add_publish_cdn_url 或者 zego_express_remove_publish_cdn_url 接口后将会收到此回调，告知添加/删除转推地址的结果
 * @param stream_id 推流的流 ID
 * @param error_code 错误码
 * @param seq 请求序列号，与调用zego_express_add_publish_cdn_url 或者 zego_express_remove_publish_cdn_url 时获得的seq对应
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_update_cdn_url_result)(const char* stream_id, zego_error error_code, zego_seq seq, void* user_context);

/**
 * 添加/删除转推 CDN 地址状态回调
 * 在 ZEGO 实时音视频云将音视频流转推到 CDN 后，如果 CDN 转推状态发生变化，例如出现转推停止或转推重试，将会收到此回调。
 * @param stream_id 流 ID
 * @param cdn_info_list 当前 CDN 正在转推的信息列表
 * @param info_count 转推 CDN 信息列表数量
 * @param user_context 用户上下文
 */
typedef void(*zego_on_publisher_relay_cdn_state_update)(const char* stream_id, const struct zego_stream_relay_cdn_info* cdn_info_list, unsigned int info_count, void* user_context);

/**
 * 开始推流
 * 可通过此接口让用户将自己本地的音视频流推送到 ZEGO 实时音视频云，同一房间的其他用户通过 streamID 就可以拉取该音视频流进行互通。
 * 在开始推流前，需要先加入房间，同房间内其他用户可通过监听 zego_on_room_stream_update 事件回调来获取该 streamID 新增。
 * 在网络质量不佳的情况下，用户推流可能出现中断， SDK 会尝试重新连接，可通过监听 zego_on_publisher_state_update 事件来获知当前推流状态以及错误信息。
 * @param stream_id 流 ID，长度不超过256的字符串，需要在整个 AppID 内全局唯一，若出现在同一个 AppID 内，不同的用户各推了一条流且流名相同 ，将会导致后推流的用户推流失败
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_start_publishing_stream(const char* stream_id, enum zego_publish_channel channel);

/**
 * 停止推流
 * 可通过此接口让用户停止发送本地的音视频流，结束通话。如果用户已经启动推流，在推新流（新的streamID）之前，必须要调用此接口停止当前流的推送，否则新流推送会返回失败。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_stop_publishing_stream(enum zego_publish_channel channel);

/**
 * 流附加信息更新
 * 可通过此接口更新当前推流的流附加信息。更新结果会通过 zego_on_publisher_update_stream_extra_info_result 回调通知。
 * @param extra_info 流附加信息，长度不超过1024的字符串。
 * @return (zego_seq)
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_set_publish_stream_extra_info(const char* extra_info, enum zego_publish_channel channel);

/**
 * 启动/更新本地预览视图
 * 用户通过调用此接口可以看到自己本地的画面。预览功能不需要先登陆房间或推流。
 * 可以通过再次调用此接口来更新本地视图和视图预览模式。
 * 可以通过调用 zego_express_set_video_mirror_mode 接口来设置预览的镜像模式，默认为开启预览镜像。
 * @param canvas 启动预览时用于显示画面的视图，视图设置为空则不进行预览
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_start_preview(struct zego_canvas* canvas, enum zego_publish_channel channel);

/**
 * 停止本地预览
 * 当本地不需要看到预览画面时可调用此接口停止预览。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_stop_preview(enum zego_publish_channel channel);

/**
 * 设置视频配置
 * 可通过此接口设置视频帧率、码率，视频采集分辨率，视频编码输出分辨率。如果不调用此接口，默认分辨率为 360p，码率为 600 kbps，帧率为 15 fps。需要在推流前设置好相关视频配置，在推流后仅支持编码分辨率和码率的修改。
 * @param video_config 视频配置，SDK 提供常用的分辨率、帧率和码率的组合值，也可自定义分辨率、帧率和码率
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_video_config(struct zego_video_config video_config, enum zego_publish_channel channel);

/**
* 获取视频配置
* 可通过此接口获取SDK当前配置的视频帧率、码率，视频采集分辨率，视频编码输出分辨率，视频编码类型。
* @param channel 推流通道，默认为主通道 zego_publish_channel_main
* @return (zego_error)
*/
ZEGOEXP_API struct zego_video_config EXP_CALL zego_express_get_video_config(enum zego_publish_channel channel);

/**
 * 设置镜像模式
 * 可调用此接口来设置本地预览视频以及推送的视频是否开启镜像模式。
 * @param mirror_mode 预览或推流的镜像模式
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_video_mirror_mode(enum zego_video_mirror_mode mirror_mode, enum zego_publish_channel channel);

/**
 * 设置采集视频的朝向（仅Android平台）
 * 此接口可设置视频的朝向方位，相比与手机正立的正向，将采集到的数据向逆时针方向分别旋转90，180或270度。旋转后会自动进行调整，以适配编码后的图像分辨率。
 * @param orientation 采集视频的朝向
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_app_orientation(enum zego_orientation orientation, enum zego_publish_channel channel);

/**
 * 设置采集视频的朝向（仅iOS平台）
 * 
 * 此接口可设置视频的朝向方位，相比正立的正向，向左/向右旋转90度，倒立等。
 * @param orientation 采集视频的朝向
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_ios_app_orientation(enum zego_ios_orientation orientation, enum zego_publish_channel channel);

/**
 * 设置音频质量配置
 * 可通过此接口设置音频采样率、码率，声道数的组合值。如果不调用此接口，默认为“标准音质”模式。仅支持推流前设置。
 * @param audio_config 音频质量配置组合值
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_audio_config(struct zego_audio_config audio_config);

/**
* 获取音频质量配置
* 可通过此接口设置音频采样率、码率，声道数的组合值。如果不调用此接口，默认为“标准音质”模式。仅支持推流前设置。
* @return (zego_audio_config) SDK 音频质量配置
*/
ZEGOEXP_API struct zego_audio_config EXP_CALL zego_express_get_audio_config();

/**
 * 停止或恢复发送音频流
 * 推流时可调用此接口实现只推视频流不推音频，本地仍会采集和处理音频，但不向网络发送音频数据。可以在推流前设置。
 * 如果在本地设置了停止发送音频流，对端可以通过监听 zego_on_remote_mic_state_update 回调收到 zego_remote_device_state_mute 的状态变更通知。
 * @param mute 是否停止发送音频流，true 表示只发送视频流不发送音频流，false 表示同时发送音频和视频流。默认为false。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_publish_stream_audio(bool mute, enum zego_publish_channel channel);

/**
 * 停止或恢复发送视频流
 * 推流时可调用此接口实现只推音频流不推视频流，本地摄像头仍能正常工作，能正常采集，预览和处理视频画面，但不向网络发送视频数据。可以在推流前设置。
 * 如果在本地设置了停止发送视频流，对端可以通过监听 zego_on_remote_camera_state_update 回调收到 zego_remote_device_state_mute 的状态变更通知。
 * @param mute 是否停止发送视频流，true 表示只发送音频流不发送视频流，false 表示同时发送音频和视频流。默认为false。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_mute_publish_stream_video(bool mute, enum zego_publish_channel channel);

/**
 * 是否开启流量控制
 *
 * @param enable 是否开启，默认为 false
 * @param properites 流量控制属性 (帧率，分辨率），参考 zego_traffic_control_property 定义。默认 zego_traffic_control_adaptive_fps
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_traffic_control(bool enable, int property);

/**
 * 设置流量控制视频码率最小值
 *
 * 当网络不足以发送这个最小值，可选择视频发送的行为
 * @param bitrate 码率，单位为kbps
 * @param mode 低于最低码率时的视频发送模式
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_min_video_bitrate_for_traffic_control(int bitrate, enum zego_traffic_control_min_video_bitrate_mode mode);

/**
 * 设置推流端采集音量
 * 此接口用于设置音频的采集音量，本端用户可控制往远端发送音频流的声音大小。可以在推流前设置。
 * @param volume 音量百分比，默认值为100
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_capture_volume(int volume);

/**
 * 增加转推至CDN的URL
 * 当需要将音视频流转推到其它指定的 CDN 时需要调用此接口进行设置。
 * @param stream_id 需要转推的流 ID
 * @param target_url CDN 转推地址，支持的转推地址格式有 rtmp，flv，hls
 * @return (zego_seq)
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_add_publish_cdn_url(const char* stream_id, const char* target_url);

/**
 * 删除转推至CDN的URL 
 * 当已经添加了某个CDN转推地址，需要停止将流停止转推至该接口时调用此接口。
 * @param stream_id 需要停止转推的流 ID
 * @param target_url CDN 转推地址，支持的转推地址格式有 rtmp，flv，hls
 * @return (zego_seq)
 */
ZEGOEXP_API zego_seq EXP_CALL zego_express_remove_publish_cdn_url(const char* stream_id, const char* target_url);

/**
 * 是否不经过Zego实时视频云服务器直接推流到CDN
 * @param enable 是否开启直推CDN，默认为 false
 * @param config CDN 配置，若为空，则使用 Zego 的后台配置
 * @return (zego_seq)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_publish_direct_to_cdn(bool enable, struct zego_cdn_config* config, enum zego_publish_channel channel);

/**
 * 设置推流水印
 * 推流前设置
 * @param is_preview_visible 是否本地预览能看见水印
 * @param watermark 水印路径以及水印布局。水印格式只能是 .jpg 或 .png 。水印布局左上角为坐标系原点，区域不能超过编码分辨率设置的大小
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_publish_watermark(bool is_preview_visible, struct zego_watermark* watermark, enum zego_publish_channel channel);

/**
 * 发送媒体增强补充信息
 * 此接口可在开发者推流传输音视频流数据同时，发送流媒体增强补充信息来同步一些其他附加信息。
 * 一般如同步音乐歌词或视频画面精准布局等场景，可选择使用发送SEI。
 * 当推流方发送SEI后，拉流方可通过监听zego_on_recv_sei的回调获取SEI内容。
 * @param buffer SEI内容。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_send_sei(const unsigned char* data, unsigned int data_length, enum zego_publish_channel channel);

/**
 * 开/关硬件编码
 * 推流时是否采用硬件编码的开关，开启硬解编码后会使用GPU进行编码，降低CPU使用率。在推流前设置才能生效，如果在推流后设置，停推后重新推流可以生效。
 * @param enable 是否开启硬件编码。true 表示开启硬件编码，false 表示关闭硬件编码。
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_hardware_encoder(bool enable);

/**
 * 设置采集时机
 * 
 * @param mode 采集时机
 * @return (zego_error)
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_set_capture_pipeline_scale_mode(enum zego_capture_pipeline_scale_mode mode);

/**
 * 注册推流状态更新回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 推流状态回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_state_update_callback(zego_on_publisher_state_update callback_func, void* user_context);

/**
 * 注册推流质量更新回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 推流质量回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_quality_update_callback(zego_on_publisher_quality_update callback_func, void* user_context);

/**
 * 流附加信息更新结果回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 流附加信息更新结果回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_update_stream_extra_info_result_callback(zego_on_publisher_update_stream_extra_info_result callback_func, void* user_context);

/**
 * 注册推流音频采集首帧回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 推流音频首帧回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_recv_audio_captured_first_frame_callback(zego_on_publisher_recv_audio_captured_first_frame callback_func, void* user_context);

/**
 * 注册视频采集首帧回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 推流视频首帧回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_recv_video_captured_first_frame_callback(zego_on_publisher_recv_video_captured_first_frame callback_func, void* user_context);

/**
 * 注册推流视频大小变更回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 推流视频分辨率变更回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_video_size_changed_callback(zego_on_publisher_video_size_changed callback_func, void* user_context);

/**
 * 注册添加/移除转推CDN URL的回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 添加/移除推流CDN URL成功失败回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_update_publish_cdn_url_callback(zego_on_publisher_update_cdn_url_result callback_func, void* user_context);

/**
 * 注册转推CDN状态更新回调
 * 
 * @param event_handler 事件通知回调
 * @param callback_func 转推CDN状态更新回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_publisher_relay_cdn_state_update_callback(zego_on_publisher_relay_cdn_state_update callback_func, void* user_context);

ZEGO_END_DECLS

#endif
