#ifndef __ZEGO_EXPRESS_ROOM_H__
#define __ZEGO_EXPRESS_ROOM_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 房间状态变化通知
 * 当房间的连接状态发生变化时将触发此回调，并告知发生变化的原因
 * @param room_id 房间 ID
 * @param state 变化后的的房间状态
 * @param error_code 错误码，详情请参考常用错误码文档
 * @param extend_data 扩展内容，默认为空
 * @param user_context 用户上下文
 */
typedef void(*zego_on_room_state_update)(const char* room_id, enum zego_room_state state, zego_error error_code, const char* extended_data, void* user_context);

/**
 * 房间内用户进出通知
 * 当有用户进入/退出房间时将触发此回调，并告知当前房间内存在的用户的列表；注意，只有在登录房间时传的 zego_room_config 中的 is_user_status_notify 参数为 true 时才会触发这个回调
 * @param room_id 房间 ID
 * @param update_type 更新类型（添加/删除）
 * @param user_list 当前房间内用户列表
 * @param user_count 用户数
 * @param user_context 用户上下文
 */
typedef void(*zego_on_room_user_update)(const char* room_id, enum zego_update_type update_type, const struct zego_user* user_list, unsigned int user_count, void* user_context);

/**
 * 房间内在线人数更新通知
 * 用户首次登录房间时会接收到房间内当前在线人数通知；后续每30S回调一次更新通知
 * @param room_id 房间ID
 * @param update_type 更新类型（添加/删除）
 * @param stream_info_list 更新的流列表
 * @param stream_info_count 更新的流数量
 * @param user_context 用户上下文
 */
typedef void(*zego_on_room_online_user_count_update)(const char* room_id, int online_user_count, void* user_context);

/**
 * 房间内流更新通知
 * 用户首次登录房间时会接收到房间内存在的所有流列表；后续当房间内新增/删除流时将触发此回调通知变更的流列表
 * @param room_id 房间ID
 * @param update_type 更新类型（添加/删除）
 * @param stream_info_list 更新的流列表
 * @param stream_info_count 更新的流数量
 * @param user_context 用户上下文
 */
typedef void(*zego_on_room_stream_update)(const char* room_id, enum zego_update_type update_type, const struct zego_stream* stream_info_list, unsigned int stream_info_count, void* user_context);

/**
 * 房间内流附加信息更新通知
 * 当房间内有用户流的附加信息更新时，本端会收到该回调
 * @param room_id 房间ID
 * @param stream_info_list 流附加信息更新的流列表
 * @param stream_info_count 流附加信息更新的流数量
 * @param user_context 用户上下文
 */
typedef void(*zego_on_room_stream_extra_info_update)(const char* room_id, const struct zego_stream* stream_info_list, unsigned int stream_info_count, void* user_context);

typedef void(*zego_on_room_send_reliable_message_result)(const char* room_id, zego_error error_code, const char* msg_type, zego_seq send_seq, zego_seq latest_seq, void* user_context);

typedef void(*zego_on_room_recv_reliable_message)(const char* room_id, const struct zego_reliable_message* message, void* user_context);

typedef void(*zego_on_room_get_reliable_message_result)(const char* room_id, zego_error error_code, zego_seq send_seq, const struct zego_reliable_message *message, void* user_context);

typedef void(*zego_on_room_update_reliable_message_info)(const char* room_id, const char* msg_type, zego_seq latest_seq, void* user_context);

/**
 * 登录房间
 * 推拉流前必须登录房间，在同一个房间内的用户可以相互通话
 * 如果由于网络质量原因导致网络临时中断，SDK 内部会自动进行重连
 * 可通过监听 zego_on_room_state_update 事件通知回调获取本端当前房间连接状态的变化情况，同时同房间其他用户会接收到 zego_on_room_user_update 回调通知。
 * @param room_id 房间ID，最大长度为256字节的字符串
 * @param user 用户对象实例，配置用户 ID、用户名
 * @return (zego_error), 错误码
 */
//ZEGOEXP_API zego_error zego_express_login_room(const char* room_id, struct zego_user user);

/**
 * 登录房间
 * 
 * 如果有较高的安全需求请使用此接口
 * 推拉流前必须登录房间，在同一个房间内的用户可以相互通话
 * 如果由于网络质量原因导致网络临时中断，SDK 内部会自动进行重连
 * 可通过监听 zego_on_room_state_update 事件通知回调获取本端当前房间连接状态的变化情况，同时同房间其他用户会接收到 zego_on_room_user_update 回调通知。
 * @param room_id 房间ID，最大长度为256字节的字符串
 * @param user 用户对象实例，配置用户 ID、用户名
 * @param config 房间高级配置，传空将使用默认配置
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_login_room(const char* room_id, struct zego_user user, struct zego_room_config* config);

/**
 * 退出房间
 * 退出房间会停止该用户的所有推拉流，调用该接口后会收到 zego_on_room_state_update 回调通知成功退出房间，同时同房间其他用户会接收到 zego_on_room_user_update 回调通知。
 * @param room_id 用户已登录的房间 ID
 * @return (zego_error), 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_logout_room(const char* room_id);


ZEGOEXP_API zego_seq EXP_CALL zego_express_send_reliable_message(const char* room_id, const char* msg_type, const char* content, unsigned int last_msg_seq);

ZEGOEXP_API zego_seq EXP_CALL zego_express_get_reliable_message(const char* room_id, const char* msg_type);

/**
 * 注册房间状态变化事件回调
 * 
 * @param callback_func 房间状态变化回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_room_state_update_callback(zego_on_room_state_update callback_func, void* user_context);

/**
 * 注册房间内用户进出通知事件回调
 * 
 * @param callback_func 用户状态更新事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_room_user_state_update_callback(zego_on_room_user_update callback_func, void* user_context);

/**
 * 注册房间内在线人数更新通知事件回调
 * 
 * @param callback_func 用户在线人数更新事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_room_online_user_count_update_callback(zego_on_room_online_user_count_update callback_func, void* user_context);

/**
 * 注册房间内流更新事件回调
 * 
 * @param callback_func 房间流添加删除事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_room_stream_update_callback(zego_on_room_stream_update callback_func, void* user_context);

/**
 * 注册房间内流附加信息更新事件回调
 * 
 * @param callback_func 房间流附加信息更新事件回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_room_stream_extra_info_update_callback(zego_on_room_stream_extra_info_update callback_func, void* user_context);

ZEGOEXP_API void EXP_CALL zego_register_room_send_reliable_message_result_callback(zego_on_room_send_reliable_message_result callback_func, void* user_context);

ZEGOEXP_API void EXP_CALL zego_register_room_get_reliable_message_result_callback(zego_on_room_get_reliable_message_result callback_func, void* user_context);

ZEGOEXP_API void EXP_CALL zego_register_room_recv_reliable_message_callback(zego_on_room_recv_reliable_message callback_func, void* user_context);

ZEGOEXP_API void EXP_CALL zego_register_room_update_reliable_message_info_callback(zego_on_room_update_reliable_message_info callback_func, void* user_context);

ZEGO_END_DECLS

#endif
